#' @include model-zelig.R
#' @include model-glm.R
zpoisson <- setRefClass("Zelig-poisson",
                        contains = "Zelig-glm",
                        fields = list(theta = "ANY"))

zpoisson$methods(
  initialize = function() {
    callSuper()
    .self$name <- "poisson"
    .self$family <- "poisson"
    .self$link <- "log"
    .self$linkinv <- eval(call(.self$family, .self$link))$linkinv
    .self$authors <- "Kosuke Imai, Gary King, Olivia Lau"
    .self$year <- 2007
    .self$category <- "count"
    .self$description <- "Poisson Regression for Event Count Dependent Variables"
    # JSON
    .self$outcome <- "discrete"
    .self$wrapper <- "poisson"
  }
)

zpoisson$methods(
  qi = function(simparam, mm) {
    eta <- simparam %*% t(mm)
    theta <- matrix(.self$linkinv(eta), nrow = nrow(simparam))
    ev <- theta
    pv <- matrix(NA, nrow = nrow(theta), ncol = ncol(theta))
    for (i in 1:ncol(theta))
      pv[, i] <- rpois(nrow(theta), lambda = theta[, i])
    return(list(ev = ev, pv = pv))
  }
)


zpoisson$methods(
  test = function(b0 = -1, b1 = 1, nsim = 1000, minx = -1, maxx = 1) {
    x.init <- mcunit.init(nsim, minx, maxx)
    
    b0 <- b0
    b1 <- b1 
    sd <- 1
    
    lambda.sim <- exp(b1 * x.init[,1] + b0)
    y.sim <- rpois(nsim, lambda.sim)
    y.true <- exp(b1 * x.init[,2] + b0)
    data <- cbind(x.init, y.sim, y.true)
    
    z <- zpoisson$new()
    callSuper(z, data)
    
  }
)

